              Filename - Filename and Pathname Manipulation
              =============================================
                        File: File - Update: 1.00
                 Author: J.G.Harston - Date: 25-Jan-2011

The Filename library provides platform-independant functions for converting
filenames between BBC, DOS/Windows and UNIX/ZIP/URL formats.

Usually a program does not, and should not, need to know what type of
filenames the underlying filing system is using. The program should accept
whatever filenames the user supplies, and pass them onto the filing system.
However, there are some cases where the actual filename format is needed
and the program needs to be able to translate filenames.

Examples would include:
* a zipfile handler, filenames within zipfiles are defined to be unix
  filenames, with "/" as directory seperators and "." as extension
  seperators.

* a program generating or recieving URLs and needing to map them to/from
  the local filesystem. URLs are defined to be unix filenames, with "/" as
  directory seperators and "." as extension seperators.

* a server application serving files on one platform where the client is
  another platform, for instance a server running on DOS/Windows serving
  files to a BBC client.

* extracting or manipulating "foreign" disk images where the program wants
  to display file information in the format the foreign system would
  display it.

Routine Summary
===============
  FNfn_bbctodos(file$)  - convert BBC filename to DOS/Windows filename
  FNfn_bbctounix(file$) - convert BBC filename to UNIX/ZIP/URL filename
  FNfn_frombbc(file$)   - convert BBC filename to local format filename
  FNfn_tobbc(file$)     - convert local format filename to BBC filename filename

  FNfn_dostobbc(file$)  - convert DOS/Windows filename to BBC filename
  FNfn_dostounix(file$) - convert DOS/Windows filename to UNIX/ZIP/URL filename
  FNfn_fromdos(file$)   - convert DOS/Windows filename to local format filename
  FNfn_todos(file$)     - convert local format filename to DOS/Windows filename

  FNfn_unixtobbc(file$) - convert UNIX/ZIP/URL filename to BBC filename
  FNfn_unixtodos(file$) - convert UNIX/ZIP/URL filename to DOS/Windows filename
  FNfn_fromunix(file$)  - convert UNIX/ZIP/URL filename to local format filename
  FNfn_tounix(file$)    - convert local format filename to UNIX/ZIP/URL filename

Requirements and Dependancies
=============================
The FNfn_from*() and FNfn_to*() functions require the global variable os%
set to the host the program is running on, such as by calling FNOS_GetEnv
in the ProgEnv library or with

    A%=0:X%=1:os%=((USR&FFF4)AND&FF00)DIV256

All the other functions have no external requirements.

FNfn_bbctodos(file$), FNfn_bbctounix(file$), FNfn_frombbc(file$), FNfn_tobbc(file$)
===================================================================================
These functions convert BBC filenames to other formats. FNfn_frombbc()
converts a BBC filename to the format of the local platform, FNfn_tobbc()
converts a local filename to a BBC filename; both according to the setting
of os%.

Examples include:
  FNfn_bbctodos("$.docs.letters/cv")  ->  \docs\letters.cv
  FNfn_bbctodos("^.info##")           ->  ..\info??
  FNfn_bbctodos(":A.text.why?")       ->  A:\text\why#

  FNfn_bbctounix("$.docs.letters/cv") ->  /docs/letters.cv
  FNfn_bbctounix("^.info##")          ->  ../info??
  FNfn_bbctounix(":A.text.why?")      ->  A:/text/why#

If running on BBC/RISC OS:
  FNfn_frombbc("$.docs.letters/cv")   ->  $.docs.letters/cv
  FNfn_frombbc("^.info##")            ->  ^.info##
  FNfn_frombbc(":A.text.why?")        ->  :A.text.why?

  FNfn_tobbc("$.docs.letters/cv")     ->  $.docs.letters/cv
  FNfn_tobbc("^.info##")              ->  ^.info##
  FNfn_tobbc(":A.text.why?")          ->  :A.text.why?

If running on DOS/Windows:
  FNfn_frombbc("$.docs.letters/cv")   ->  \docs\letters.cv
  FNfn_frombbc("^.info##")            ->  ..\info??
  FNfn_frombbc(":A.text.why?")        ->  A:\text\why#

  FNfn_tobbc("\docs\letters.cv")      ->  $.docs.letters/cv
  FNfn_tobbc("..\info??")             ->  ^.info##
  FNfn_tobbc("A:\text\why#")          ->  :A.text.why?

If running on UNIX:
  FNfn_frombbc("$.docs.letters/cv")   ->  /docs/letters.cv
  FNfn_frombbc("^.info##")            ->  ../info??
  FNfn_frombbc(":A.text.why?")        ->  A:/text/why#

  FNfn_tobbc("/docs/letters.cv")      ->  $.docs.letters/cv
  FNfn_tobbc("../info??")             ->  ^.info##
  FNfn_tobbc("A:/text/why#")          ->  :A.text.why?

FNfn_dostobbc(file$), FNfn_dostounix(file$), FNfn_fromdos(file$), FNfn_todos(file$)
===================================================================================
These functions convert DOS/Windows filenames to other formats.
FNfn_fromdos() converts a DOS/Windows filename to the format of the local
platform, FNfn_todos() converts a local filename to a DOS/Windows filename;
both according to the setting of os%.

Examples include:
  FNfn_dostobbc("\docs\letters.cv")   ->  $.docs.letters/cv
  FNfn_dostobbc("..\info??")          ->  ^.info##
  FNfn_dostobbc("A:\text\why#")       ->  :A.text.why?

  FNfn_dostounix("\docs\letters.cv")  ->  /docs/letters.cv
  FNfn_dostounix("..\info??")         ->  ../info??
  FNfn_dostounix("A:\text\why#")      ->  A:/text/why#

If running on BBC/RISC OS:
  FNfn_fromdos("\docs\letters.cv")    ->  $.docs.letters/cv
  FNfn_fromdos("..\info??")           ->  ^.info##
  FNfn_fromdos("A:\text\why#")        ->  :A.text.why?

  FNfn_todos("$.docs.letters/cv")     ->  \docs\letters.cv
  FNfn_todos("^.info##")              ->  ..\info??
  FNfn_todos(":A.text.why?")          ->  A:\text\why#

If running on DOS/Windows:
  FNfn_fromdos("\docs\letters.cv")    ->  \docs\letters/cv
  FNfn_fromdos("..\info??")           ->  ..\info??
  FNfn_fromdos("A:\text\why#")        ->  A:\text\why?

  FNfn_todos("\docs\letters.cv")      ->  \docs\letters.cv
  FNfn_todos("..\info??")             ->  ..\info??
  FNfn_todos("A:\text\why#")          ->  A:\text\why#

If running on UNIX:
  FNfn_fromdos("\docs\letters.cv")    ->  /docs/letters.cv
  FNfn_fromdos("..\info??")           ->  ../info??
  FNfn_fromdos("A:\text\why#")        ->  A:/text/why#

  FNfn_todos("/docs/letters.cv")      ->  \docs\letters.cv
  FNfn_todos("../info??")             ->  ..\info??
  FNfn_todos("A:/text/why#")          ->  A:\text\why#

FNfn_unixtobbc(file$), FNfn_unixtodos(file$), FNfn_fromunix(file$), FNfn_tounix(file$)
======================================================================================
These functions convert UNIX/ZIP/URL filenames to other formats.
FNfn_fromunix() converts a UNIX/ZIP/URL filename to the format of the local
platform, FNfn_tounix() converts a local filename to a UNIX/ZIP/URL
filename; both according to the setting of os%.

Examples include:
  FNfn_unixtobbc("/docs/letters.cv")  ->  $.docs.letters/cv
  FNfn_unixtobbc("../info??")         ->  ^.info##
  FNfn_unixtobbc("A:/text/why#")      ->  :A.text.why?

  FNfn_unixtodos("\docs\letters.cv")  ->  \docs\letters.cv
  FNfn_unixtodos("..\info??")         ->  ..\info??
  FNfn_unixtodos("A:\text\why#")      ->  A:\text\why#

If running on BBC/RISC OS:
  FNfn_fromunix("/docs/letters.cv")   ->  $.docs.letters/cv
  FNfn_fromunix("../info??")          ->  ^.info##
  FNfn_fromunix("A:/text/why#")       ->  :A.text.why?

  FNfn_tounix("$.docs.letters/cv")    ->  /docs/letters.cv
  FNfn_tounix("^.info##")             ->  ../info??
  FNfn_tounix(":A.text.why?")         ->  A:/text/why#

If running on DOS/Windows:
  FNfn_fromunix("/docs/letters.cv")   ->  \docs\letters/cv
  FNfn_fromunix("../info??")          ->  ..\info??
  FNfn_fromunix("A:/text/why#")       ->  A:\text\why?

  FNfn_tounix("\docs\letters.cv")     ->  /docs/letters.cv
  FNfn_tounix("..\info??")            ->  ../info??
  FNfn_tounix("A:\text\why#")         ->  A:/text/why#

If running on UNIX:
  FNfn_fromunix("/docs/letters.cv")   ->  /docs/letters.cv
  FNfn_fromunix("../info??")          ->  ../info??
  FNfn_fromunix("A:/text/why#")       ->  A:/text/why#

  FNfn_tounix("/docs/letters.cv")      ->  /docs/letters.cv
  FNfn_tounix("../info??")             ->  ../info??
  FNfn_tounix("A:/text/why#")          ->  A:/text/why#

Technical information
=====================
Different platforms use different characters to indicate different filename
components such as directory and extension seperators, drive specifiers,
current, root and parent directories.

                      BBC     DOS/Windows       UNIX/ZIP/URL
---------------------------------------------------------------
Drive                 :d            d:
Directory seperator   .             \                /
Extension seperator   /             .                .
Root directory        $             \                /
Parent directory      ^             ..               ..
Current Directory     @             .                .
Library Directory     %             
User Directory        &             %USERPROFILE%    $HOME$
Wild Character        #             ?                ?
Wild String           *             *                *

The simplest conversion is between UNIX and DOS filenames; other than the
'/' and '\' directory seperators all other characters used are identical.
Conversion is simply an issue of swapping '/' and '\' characters.

The BBC uses '#' as a single-character wildcard, DOS and UNIX use '?'.
Conversely, on the BBC '?' is an ordinary character and on DOS and UNIX '#'
is an ordinary character, so thay are swapped.

The BBC uses '.' as a directory seperator and '/' as an extension
seperator; DOS and UNIX use '.' as an extension seperator, DOS uses '\' as
a directory seperator and UNIX uses '/'. '.' and '/' are swapped when
converting between BBC and UNIX. BBC '.', '\', '/' are swapped  with DOS
'\', '/', '.' when converting between BBC and DOS.

The functions in this library do not provide a perfect translation mapping,
the BBC has more illegal filename characters than other platforms which
makes a bidirection mapping difficult. For example, the DOS filename
"$TEMP$.$$$" cannot be translated to a BBC filename and then have a perfect
one-to-one mapping back to the same DOS filename. If support for such more
complex filenames are needed a more advanced Filename library should be
used.

Version History
===============
1.00 25-Jan-2011 Initial version based on earlier embedded code.
